#include <stdio.h>
#include <jpeglib.h>
#include <jpegint.h>
#include "jerror.h"
#include "cdjpeg.h"
#include <time.h>

LOCAL(cjpeg_source_ptr)
select_file_type (j_compress_ptr cinfo, FILE *infile)
{
  int c;

  if ((c = getc(infile)) == EOF)
    ERREXIT(cinfo, JERR_INPUT_EMPTY);
  if (ungetc(c, infile) == EOF)
    ERREXIT(cinfo, JERR_UNGETC_FAILED);

  switch (c) {
  case 'B':
    return jinit_read_bmp(cinfo);
    ERREXIT(cinfo, JERR_UNKNOWN_FORMAT);
    break;
  }
  return NULL;                  /* suppress compiler warnings */
}

/******************** JPEG COMPRESSION SAMPLE INTERFACE *******************/

/*
 * Sample routine for JPEG compression.
 */

GLOBAL(void)
write_JPEG_file (char *inputfile, char* outputfile, int quality)
{
  clock_t t;
  /* This struct contains the JPEG compression parameters and pointers. We refer
   * to any one struct as a "JPEG object".
   */
  struct jpeg_compress_struct cinfo;

  /* This struct represents a JPEG error handler.
   */
  struct jpeg_error_mgr jerr;

  /* Input and output file */
  FILE *infile;
  FILE *outfile;                

  /* Other stuff */
  int file_index;
  unsigned char *outbuffer = NULL;
  unsigned long outsize = 0;
  JDIMENSION num_scanlines;
  cjpeg_source_ptr src_mgr;

  /* Step 1: allocate and initialize JPEG compression object */
  cinfo.err = jpeg_std_error(&jerr);
  /* Now we can initialize the JPEG compression object. */
  jpeg_create_compress(&cinfo);

  // Arbitrary initialize the compression parameter. They will be overwritten later.
  cinfo.in_color_space = JCS_RGB; /* arbitrary guess */
  jpeg_set_defaults(&cinfo);

  /* Step 2: specify data destination (eg, a file) */
  /* Note: steps 2 and 3 can be done in either order. */

  /* Here we use the library-supplied code to send compressed data to a
   * stdio stream.  You can also write your own code to do something else.
   * VERY IMPORTANT: use "b" option to fopen() if you are on a machine that
   * requires it in order to write binary files.
   */
  if ((infile = fopen(inputfile, "rb")) == NULL) {
    fprintf(stderr, "can't open %s\n", inputfile);
    exit(1);
  }

  if ((outfile = fopen(outputfile, "wb")) == NULL) {
    fprintf(stderr, "can't open %s\n", outputfile);
    exit(1);
  }

  jpeg_stdio_dest(&cinfo, outfile);

  /* Step 3: set parameters for compression */

  /* Figure out the input file format, and set up to read it. */
  src_mgr = select_file_type(&cinfo, infile);
  src_mgr->input_file = infile;
  /* Read the input file header to obtain file size & colorspace. */
  (*src_mgr->start_input) (&cinfo, src_mgr);
  /* Now that we know input colorspace, fix colorspace-dependent defaults */
  jpeg_default_colorspace(&cinfo);
  jpeg_set_quality(&cinfo, quality, TRUE /* limit to baseline-JPEG values */);

  /* Step 4: Start compressor */

  /* TRUE ensures that we will write a complete interchange-JPEG file.
   * Pass TRUE unless you are very sure of what you're doing.
   */
  jpeg_start_compress(&cinfo, TRUE);

  t = clock();
  /* Step 5: while (scan lines remain to be written) */
  /*           jpeg_write_scanlines(...); */
  while (cinfo.next_scanline < cinfo.image_height) {
    num_scanlines = (*src_mgr->get_pixel_rows) (&cinfo, src_mgr);
    (void) jpeg_write_scanlines(&cinfo, src_mgr->buffer, num_scanlines);
  }

  /* Step 6: Finish compression */
  /* Finish compression and release memory */
  (*src_mgr->finish_input) (&cinfo, src_mgr);
  jpeg_finish_compress(&cinfo);
  t = clock() - t;
  jpeg_destroy_compress(&cinfo);



  double time_taken = (((double)t)/CLOCKS_PER_SEC)*1000.0;
  printf("compress took %f ms to execute \n", time_taken);

  /* After finish_compress, we can close the input and output file. */
  fclose(infile);
  fclose(outfile);

  /* Step 7: release JPEG compression object */

  /* This is an important step since it will release a good deal of memory. */
  jpeg_destroy_compress(&cinfo);

  /* And we're done! */
}


int main(int argc,char **argv)
{ 
  char* fileraw = "sphere.bmp";
  char* output = "output.jpg";
  write_JPEG_file(fileraw, output, 40);

  return 0;
}
